@@ -374,8 +374,6 @@ EXCLUDE = (
     "lib/neatogen/stress.h",
     "lib/neatogen/stuff.c",
     "lib/neatogen/voronoi.c",
-    "lib/ortho/fPQ.c",
-    "lib/ortho/fPQ.h",
     "lib/ortho/maze.c",
     "lib/ortho/maze.h",
     "lib/ortho/ortho.c",
@@ -404,7 +402,6 @@ EXCLUDE = (
     "lib/pathplan/solvers.c",
     "lib/pathplan/solvers.h",
     "lib/pathplan/triang.c",
-    "lib/pathplan/util.c",
     "lib/pathplan/vis.h",
     "lib/pathplan/visibility.c",
     "lib/pathplan/vispath.h",
