@@ -92,12 +92,12 @@ def checksum(path: Path) -> Path:
 
 def is_macos_artifact(name: str) -> bool:
     """is this a deployment artifact for macOS?"""
-    return re.search(r"\bDarwin\b", name) is not None
+    return name.startswith("Darwin_")
 
 
 def is_windows_artifact(name: str) -> bool:
     """is this a deployment artifact for Windows?"""
-    return re.search(r"\bwindows\b", name) is not None
+    return name.startswith("windows_")
 
 
 def get_format(path: Path) -> str:
@@ -203,19 +203,19 @@ def main() -> int:
         "fedora42-build": [f"fedora_42_graphviz-{version}-rpms.tar.xz"],
         "fedora43-build": [f"fedora_43_graphviz-{version}-rpms.tar.xz"],
         "rocky8-build": [f"rocky_8.9_graphviz-{version}-rpms.tar.xz"],
-        "rocky9-build": [f"rocky_9.3_graphviz-{version}-rpms.tar.xz"],
+        "rocky9-build": [f"rocky_9.7_graphviz-{version}-rpms.tar.xz"],
+        "rocky10-build": [f"rocky_10.1_graphviz-{version}-rpms.tar.xz"],
         "fedora41-cmake-build": [f"fedora_41_graphviz-{version}-cmake.rpm"],
         "fedora42-cmake-build": [f"fedora_42_graphviz-{version}-cmake.rpm"],
         "fedora43-cmake-build": [f"fedora_43_graphviz-{version}-cmake.rpm"],
         "rocky8-cmake-build": [f"rocky_8.9_graphviz-{version}-cmake.rpm"],
-        "rocky9-cmake-build": [f"rocky_9.3_graphviz-{version}-cmake.rpm"],
+        "rocky9-cmake-build": [f"rocky_9.7_graphviz-{version}-cmake.rpm"],
+        "rocky10-cmake-build": [f"rocky_10.1_graphviz-{version}-cmake.rpm"],
         "ubuntu-22.04-build": [f"ubuntu_22.04_graphviz-{version}-debs.tar.xz"],
         "ubuntu-24.04-build": [f"ubuntu_24.04_graphviz-{version}-debs.tar.xz"],
-        "ubuntu-25.04-build": [f"ubuntu_25.04_graphviz-{version}-debs.tar.xz"],
         "ubuntu-25.10-build": [f"ubuntu_25.10_graphviz-{version}-debs.tar.xz"],
         "ubuntu-22.04-cmake-build": [f"ubuntu_22.04_graphviz-{version}-cmake.deb"],
         "ubuntu-24.04-cmake-build": [f"ubuntu_24.04_graphviz-{version}-cmake.deb"],
-        "ubuntu-25.04-cmake-build": [f"ubuntu_25.04_graphviz-{version}-cmake.deb"],
         "ubuntu-25.10-cmake-build": [f"ubuntu_25.10_graphviz-{version}-cmake.deb"],
         "macos-autotools-build": [f"Darwin_23.6.0_graphviz-{version}-arm64.pkg"],
         "macos-cmake-build": [f"Darwin_23.6.0_Graphviz-{version}-Darwin.zip"],
