@@ -43,13 +43,10 @@ static ssize_t outfn(void *sp, const char *buf, size_t nbyte, void *dp)
 int run_gvpr(Agraph_t * srcGraph, size_t argc, char *argv[]) {
     int rv = 1;
     gvpropts opts = {0};
-    Agraph_t *gs[2];
     static int count;
     agxbuf buf = {0};
 
-    gs[0] = srcGraph;
-    gs[1] = 0;
-    opts.ingraphs = gs;
+    opts.ingraphs = (Agraph_t *[]){srcGraph, NULL};
     opts.out = outfn;
     opts.err = outfn;
     opts.flags = GV_USE_OUTGRAPH;
