@@ -18,6 +18,8 @@
  * Updated by Emden Gansner
  */
 
+#include "config.h"
+
 #include <assert.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/ingraphs.h>
@@ -210,19 +212,18 @@ static void init(int argc, char *argv[]) {
     Inputs = argv;
 }
 
-DEFINE_LIST(node_stack, Agnode_t *)
-static node_stack_t Stk;
+static LIST(Agnode_t *) Stk;
 
 static void push(Agnode_t *np) {
   Node_mark(np) = -1;
-  node_stack_push_back(&Stk, np);
+  LIST_PUSH_BACK(&Stk, np);
 }
 
 static Agnode_t *pop(void) {
-  if (node_stack_is_empty(&Stk)) {
+  if (LIST_IS_EMPTY(&Stk)) {
     return NULL;
   }
-  return node_stack_pop_back(&Stk);
+  return LIST_POP_BACK(&Stk);
 }
 
 static int dfs(Agraph_t *g, Agnode_t *n, Agraph_t *out) {
@@ -338,7 +339,7 @@ static void deriveClusters(Agraph_t *dg, Agraph_t *g) {
     if (is_a_cluster(subg)) {
       Agnode_t *const dn = agnode(dg, agnameof(subg), 1);
       agbindrec(dn, "nodeinfo", sizeof(nodeinfo_t), true);
-      ND_ptr(dn) = (Agobj_t *)subg;
+      ND_ptr(dn) = &subg->base;
       for (Agnode_t *n = agfstnode(subg); n; n = agnxtnode(subg, n)) {
         if (ND_ptr(n)) {
           fprintf(stderr,
@@ -346,7 +347,7 @@ static void deriveClusters(Agraph_t *dg, Agraph_t *g) {
                   "\"%s\" and \"%s\"\n",
                   agnameof(n), agnameof(subg), agnameof(ND_dn(n)));
         }
-        ND_ptr(n) = (Agobj_t *)dn;
+        ND_ptr(n) = &dn->base;
       }
     } else {
       deriveClusters(dg, subg);
@@ -368,8 +369,8 @@ static Agraph_t *deriveGraph(Agraph_t *g) {
       continue;
     Agnode_t *const dn = agnode(dg, agnameof(n), 1);
     agbindrec(dn, "nodeinfo", sizeof(nodeinfo_t), true);
-    ND_ptr(dn) = (Agobj_t *)n;
-    ND_ptr(n) = (Agobj_t *)dn;
+    ND_ptr(dn) = &n->base;
+    ND_ptr(n) = &dn->base;
   }
 
   for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -402,18 +403,8 @@ static void unionNodes(Agraph_t *dg, Agraph_t *g) {
 }
 
 static int cmp(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because `agnnodes` uses
-// non-const pointers.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  Agraph_t **p0 = (Agraph_t **)x;
-  Agraph_t **p1 = (Agraph_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
+  Agraph_t *const *const p0 = x;
+  Agraph_t *const *const p1 = y;
   const int n0 = agnnodes(*p0);
   const int n1 = agnnodes(*p1);
   if (n0 < n1) {
@@ -739,7 +730,7 @@ int main(int argc, char *argv[]) {
     agclose(g);
   }
 
-  node_stack_free(&Stk);
+  LIST_FREE(&Stk);
 
   graphviz_exit(r);
 }
