@@ -8,6 +8,8 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include "config.h"
+
 #include <dotgen/dot.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -107,8 +109,8 @@ void zapinlist(elist * L, edge_t * e)
 void delete_fast_edge(edge_t * e)
 {
     assert(e != NULL);
-    zapinlist(&(ND_out(agtail(e))), e);
-    zapinlist(&(ND_in(aghead(e))), e);
+    zapinlist(&ND_out(agtail(e)), e);
+    zapinlist(&ND_in(aghead(e)), e);
 }
 
 void other_edge(edge_t * e)
@@ -226,50 +228,6 @@ void delete_flat_edge(edge_t * e)
     zapinlist(&ND_flat_in(aghead(e)), e);
 }
 
-#ifdef DEBUG
-static char *NAME(node_t * n)
-{
-    static char buf[20];
-    if (ND_node_type(n) == NORMAL)
-	return agnameof(n);
-    snprintf(buf, sizeof(buf), "V%p", n);
-    return buf;
-}
-
-void fastgr(graph_t * g)
-{
-    int i, j;
-    node_t *n, *w;
-    edge_t *e, *f;
-
-    for (n = GD_nlist(g); n; n = ND_next(n)) {
-	fprintf(stderr, "%s %d: (", NAME(n), ND_rank(n));
-	for (i = 0; (e = ND_out(n).list[i]); i++) {
-	    fprintf(stderr, " %s:%d", NAME(aghead(e)), ED_count(e));
-	    w = aghead(e);
-	    if (g == agroot(g)) {
-		for (j = 0; (f = ND_in(w).list[j]); j++)
-		    if (e == f)
-			break;
-		assert(f != NULL);
-	    }
-	}
-	fprintf(stderr, " ) (");
-	for (i = 0; (e = ND_in(n).list[i]); i++) {
-	    fprintf(stderr, " %s:%d", NAME(agtail(e)), ED_count(e));
-	    w = agtail(e);
-	    if (g == agroot(g)) {
-		for (j = 0; (f = ND_out(w).list[j]); j++)
-		    if (e == f)
-			break;
-		assert(f != NULL);
-	    }
-	}
-	fprintf(stderr, " )\n");
-    }
-}
-#endif
-
 static void 
 basic_merge(edge_t * e, edge_t * rep)
 {
