@@ -16,6 +16,8 @@
  * because mincross may compare nodes in different clusters.
  */
 
+#include "config.h"
+
 #include <assert.h>
 #include <cgraph/cgraph.h>
 #include <dotgen/dot.h>
@@ -145,7 +147,6 @@ static void free_matrix(adjmatrix_t * p);
 static int ordercmpf(const void *, const void *);
 static int64_t ncross(void);
 #ifdef DEBUG
-void check_rs(graph_t * g, int null_ok);
 void check_order(void);
 void check_vlists(graph_t * g);
 void node_in_root_vlist(node_t * n);
@@ -162,86 +163,6 @@ static edge_t **TE_list;
 static int *TI_list;
 static bool ReMincross;
 
-#if defined(DEBUG) && DEBUG > 1
-static void indent(graph_t* g)
-{
-  if (g->parent) {
-    fprintf (stderr, "  ");
-    indent(g->parent);
-  }
-}
-
-/// @param stream Output stream to write to
-static void nname(node_t *v, FILE *stream) {
-	if (ND_node_type(v)) {
-		if (ND_ranktype(v) == CLUSTER)
-			fprintf(stream, "v%s_%p", agnameof(ND_clust(v)), v);
-		else
-			fprintf(stream, "v_%p", v);
-	} else
-		fputs(agnameof(v), stream);
-}
-static void dumpg (graph_t* g)
-{
-    edge_t* e;
-
-    fprintf (stderr, "digraph A {\n");
-    for (int r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	fprintf (stderr, "  subgraph {rank=same  ");
-	const char *trailer = " }\n";
-	for (int i = 0; i < GD_rank(g)[r].n; i++) {
-	  node_t *const v = GD_rank(g)[r].v[i];
-          if (i > 0) {
- 	    fputs(" -> ", stderr);
- 	    trailer = " [style=invis]}\n";
-          }
-          nname(v, stderr);
-        }
-        fputs(trailer, stderr);
-    }
-    for (int r = GD_minrank(g); r < GD_maxrank(g); r++) {
-	for (int i = 0; i < GD_rank(g)[r].n; i++) {
-	  node_t *const v = GD_rank(g)[r].v[i];
-	  for (int j = 0; (e = ND_out(v).list[j]); j++) {
-             nname(v, stderr);
-             fputs(" -> ", stderr);
-             nname(aghead(e), stderr);
-             fputc('\n', stderr);
-          }
-        }
-    }
-    fprintf (stderr, "}\n");
-}
-static void dumpr (graph_t* g, int edges)
-{
-    int j, i, r;
-    node_t* v;
-    edge_t* e;
-
-    for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	fprintf (stderr, "[%d] ", r);
-	for (i = 0; i < GD_rank(g)[r].n; i++) {
-	  v = GD_rank(g)[r].v[i];
- 	  nname(v, stderr);
- 	  fprintf(stderr, "(%.02f,%d) ", saveorder(v),ND_order(v));
-        }
-	fprintf (stderr, "\n");
-    }
-    if (edges == 0) return;
-    for (r = GD_minrank(g); r < GD_maxrank(g); r++) {
-	for (i = 0; i < GD_rank(g)[r].n; i++) {
-	  v = GD_rank(g)[r].v[i];
-	  for (j = 0; (e = ND_out(v).list[j]); j++) {
-             nname(v, stderr);
-             fputs(" -> ", stderr);
-             nname(aghead(e), stderr);
-             fputc('\n', stderr);
-          }
-        }
-    }
-}
-#endif
-
 typedef struct {
     Agrec_t h;
     int x, lo, hi;
@@ -278,23 +199,23 @@ findSource (Agraph_t* g, Agraph_t* sg)
     return NULL;
 }
 
-static int
-topsort (Agraph_t* g, Agraph_t* sg, Agnode_t** arr)
-{
+typedef LIST(node_t *) nodes_t;
+
+static nodes_t topsort(Agraph_t *g, Agraph_t *sg) {
     Agnode_t* n;
     Agedge_t* e;
     Agedge_t* nxte;
-    int cnt = 0;
+    nodes_t arr = {0};
 
     while ((n = findSource(g, sg))) {
-	arr[cnt++] = ND_np(n); 
+	LIST_APPEND(&arr, ND_np(n)); 
 	agdelnode(sg, n);
 	for (e = agfstout(g, n); e; e = nxte) {
 	    nxte = agnxtout(g, e); 
 	    agdeledge(g, e);
 	}
     }
-    return cnt;
+    return arr;
 }
 
 static int
@@ -323,7 +244,6 @@ getComp (graph_t* g, node_t* n, graph_t* comp, int* indices)
 static void
 fixLabelOrder (graph_t* g, rank_t* rk)
 {
-    int cnt;
     bool haveBackedge = false;
     Agraph_t* sg;
     Agnode_t* n;
@@ -345,25 +265,23 @@ fixLabelOrder (graph_t* g, rank_t* rk)
     if (!haveBackedge) return;
     
     sg = agsubg(g, "comp", 1);
-    Agnode_t **arr = gv_calloc(agnnodes(g), sizeof(Agnode_t*));
     int *indices = gv_calloc(agnnodes(g), sizeof(int));
 
     for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
 	if (ND_x(n) || agdegree(g,n,1,1) == 0) continue;
 	if (getComp(g, n, sg, indices)) {
-	    int i, sz = agnnodes(sg);
-	    cnt = topsort (g, sg, arr);
-	    assert (cnt == sz);
-	    qsort(indices, cnt, sizeof(int), ordercmpf);
-	    for (i = 0; i < sz; i++) {
-		ND_order(arr[i]) = indices[i];
-		rk->v[indices[i]] = arr[i];
+	    nodes_t arr = topsort(g, sg);
+	    assert(LIST_SIZE(&arr) == (size_t)agnnodes(sg));
+	    qsort(indices, LIST_SIZE(&arr), sizeof(int), ordercmpf);
+	    for (size_t i = 0; i < LIST_SIZE(&arr); i++) {
+		ND_order(LIST_GET(&arr, i)) = indices[i];
+		rk->v[indices[i]] = LIST_GET(&arr, i);
 	    }
+	    LIST_FREE(&arr);
        }
        emptyComp(sg);
     }
     free(indices);
-    free (arr);
 }
 
 /* Check that the ordering of labels for flat edges is consistent. 
@@ -507,7 +425,7 @@ static int betweenclust(edge_t * e)
 {
     while (ED_to_orig(e))
 	e = ED_to_orig(e);
-    return (ND_clust(agtail(e)) != ND_clust(aghead(e)));
+    return ND_clust(agtail(e)) != ND_clust(aghead(e));
 }
 
 static void do_ordering_node(graph_t *g, node_t *n, bool outflag) {
@@ -529,8 +447,7 @@ static void do_ordering_node(graph_t *g, node_t *n, bool outflag) {
     }
     if (ne <= 1)
 	return;
-    /* write null terminator at end of list.
-       requires +1 in TE_list alloccation */
+    // Write null terminator at end of list. Requires +1 in TE_list allocation.
     sortlist[ne] = 0;
     qsort(sortlist, ne, sizeof(sortlist[0]), edgeidcmpf);
     for (ne = 1; (f = sortlist[ne]); ne++) {
@@ -690,7 +607,7 @@ static int out_cross(node_t * v, node_t * w)
 
 	for (e1 = ND_out(v).list; *e1; e1++) {
 	    t = ND_order(aghead(*e1)) - inv;
-	    if (t > 0 || (t == 0 && (ED_head_port(*e1)).p.x > (ED_head_port(*e2)).p.x))
+	    if (t > 0 || (t == 0 && ED_head_port(*e1).p.x > ED_head_port(*e2).p.x))
 		cross += ED_xpenalty(*e1) * cnt;
 	}
     }
@@ -1399,8 +1316,6 @@ static bool constraining_flat_edge(Agraph_t *g, Agedge_t *e) {
   return true;
 }
 
-typedef LIST(node_t *) nodes_t;
-
 /* construct nodes reachable from 'here' in post-order.
 * This is the same as doing a topological sort in reverse order.
 */
@@ -1761,18 +1676,8 @@ static bool medians(graph_t * g, int r0, int r1)
 }
 
 static int nodeposcmpf(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  node_t **n0 = (node_t **)x;
-  node_t **n1 = (node_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
+  node_t *const *const n0 = x;
+  node_t *const *const n1 = y;
   if (ND_order(*n0) < ND_order(*n1)) {
     return -1;
   }
@@ -1783,18 +1688,8 @@ static int nodeposcmpf(const void *x, const void *y) {
 }
 
 static int edgeidcmpf(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  edge_t **e0 = (edge_t **)x;
-  edge_t **e1 = (edge_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
+  edge_t *const *const e0 = x;
+  edge_t *const *const e1 = y;
   if (AGSEQ(*e0) < AGSEQ(*e1)) {
     return -1;
   }
@@ -1815,7 +1710,7 @@ static int edgeidcmpf(const void *x, const void *y) {
 #define C_SS		2
 #define C_VV		4
 
-static int table[NTYPES][NTYPES] = {
+static const int table[NTYPES][NTYPES] = {
     /* ordinary */ {C_EE, C_EE, C_EE},
     /* singleton */ {C_EE, C_SS, C_VS},
     /* virtual */ {C_EE, C_VS, C_VV}
@@ -1846,32 +1741,6 @@ void virtual_weight(edge_t * e)
 }
 
 #ifdef DEBUG
-void check_rs(graph_t * g, int null_ok)
-{
-    int i, r;
-    node_t *v, *prev;
-
-    fprintf(stderr, "\n\n%s:\n", agnameof(g));
-    for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	fprintf(stderr, "%d: ", r);
-	prev = NULL;
-	for (i = 0; i < GD_rank(g)[r].n; i++) {
-	    v = GD_rank(g)[r].v[i];
-	    if (v == NULL) {
-		fprintf(stderr, "NULL\t");
-		if (!null_ok)
-		    abort();
-	    } else {
-		fprintf(stderr, "%s(%f)\t", agnameof(v), ND_mval(v));
-		assert(ND_rank(v) == r);
-		assert(v != prev);
-		prev = v;
-	    }
-	}
-	fprintf(stderr, "\n");
-    }
-}
-
 void check_order(void)
 {
     int i, r;
@@ -1905,25 +1774,6 @@ static void mincross_options(graph_t * g)
 }
 
 #ifdef DEBUG
-void check_exchange(node_t * v, node_t * w)
-{
-    int i, r;
-    node_t *u;
-
-    if (ND_clust(v) == NULL && ND_clust(w) == NULL)
-	return;
-    assert(ND_clust(v) == NULL || ND_clust(w) == NULL);
-    assert(ND_rank(v) == ND_rank(w));
-    assert(ND_order(v) < ND_order(w));
-    r = ND_rank(v);
-
-    for (i = ND_order(v) + 1; i < ND_order(w); i++) {
-	u = GD_rank(dot_root(v))[r].v[i];
-	if (ND_clust(u))
-	    abort();
-    }
-}
-
 void check_vlists(graph_t * g)
 {
     int c, i, j, r;
