@@ -30,9 +30,10 @@
 #include <unistd.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
-#include <util/exit.h>
+#include <util/list.h>
 #include <util/prisize_t.h>
 #include <util/startswith.h>
+#include <util/strview.h>
 #include <util/unreachable.h>
 
 static int isedge(Agobj_t *obj) {
@@ -96,7 +97,7 @@ static char *bbOf(Expr_t *pgm, char *pt, bool getll) {
     if (getll) {
       size_t len = (size_t)(p - pt);
       char *const s = exstralloc(pgm, len + 1);
-      strncpy(s, pt, len);
+      memcpy(s, pt, len);
       s[len] = '\0';
       return s;
     }
@@ -116,7 +117,7 @@ static char *xyOf(Expr_t *pgm, char *pt, bool getx) {
     if (getx) {
       size_t len = (size_t)(p - pt);
       char *const v = exstralloc(pgm, len + 1);
-      strncpy(v, pt, len);
+      memcpy(v, pt, len);
       v[len] = '\0';
       return v;
     }
@@ -149,31 +150,6 @@ static int posOf(Agnode_t *np, int idx, double *v) {
     return -1;
 }
 
-#if defined(DEBUG) && DEBUG > 1
-static char *symName(Expr_t *ex, int op) {
-  if (op >= MINNAME && op <= MAXNAME)
-    return gprnames[op];
-  else {
-    // calculate how much space we need to construct a name
-    int bytes = vsnprintf(NULL, 0, "<unknown (%d)>", op);
-    if (bytes < 0) {
-      fprintf(stderr, "%s: vsnprintf failure\n", __func__);
-      graphviz_exit(EXIT_FAILURE);
-    }
-
-    // construct a managed buffer to store this name
-    char *s = vmalloc(ex->ve, (size_t)bytes + 1);
-
-    // make the name
-    if (s != NULL) {
-      snprintf(s, (size_t)bytes + 1, "<unknown (%d)>", op);
-    }
-
-    return s;
-  }
-}
-#endif
-
 /* Convert string argument to graph to type of graph desired.
  *   u => undirected
  *   d => directed
@@ -241,7 +217,7 @@ static Agobj_t *deref(Expr_t *pgm, Exnode_t *x, Exref_t *ref, Agobj_t *objp,
       agxbfree(&xb);
       return ptr;
     } else
-      return deref(pgm, x, ref->next, (Agobj_t *)ptr, state);
+      return deref(pgm, x, ref->next, ptr, state);
   } else
     switch (ref->symbol->index) { /* sym->lex == ID */
     case V_outgraph:
@@ -487,13 +463,18 @@ static int lookup(Expr_t *pgm, Agobj_t *objp, Exid_t *sym, Extype_t *v) {
   return 0;
 }
 
-// return value associated with $n
-static char *getArg(int n, Gpr_t *state) {
-  if (n >= state->argc) {
-    exerror("program references ARGV[%d] - undefined", n);
-    return 0;
+/// get value associated with $n
+///
+/// @param out [out] Found argument on success
+/// @return 0 on success
+static int getArg(long long n, Gpr_t *state, strview_t *out) {
+  assert(out != NULL);
+  if (n < 0 || (unsigned long long)n >= LIST_SIZE(&state->args)) {
+    exerror("program references ARGV[%lld] - undefined", n);
+    return -1;
   }
-  return state->argv[n];
+  *out = LIST_GET(&state->args, (size_t)n);
+  return 0;
 }
 
 static int setDfltAttr(Agraph_t *gp, char *k, char *name, char *value) {
@@ -600,6 +581,9 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
     switch (sym->index) {
     case F_graph:
       gp = openG(args[0].string, xargs(args[1].string));
+      if (gp != NULL) {
+        LIST_APPEND(&state->open_graphs, gp);
+      }
       v.integer = ptr2int(gp);
       break;
     case F_subg:
@@ -1182,8 +1166,14 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
       } else if (objp == state->curobj) {
         if (!(v.integer = deleteObj(gp, objp)))
           state->curobj = NULL;
-      } else
+      } else {
+        // Drop this from the list of managed open graphs. If it was not open,
+        // let this silently fail.
+        if (AGTYPE(objp) == AGRAPH) {
+          LIST_REMOVE(&state->open_graphs, (Agraph_t *)objp);
+        }
         v.integer = deleteObj(gp, objp);
+      }
       break;
     case F_lock:
       gp = int2ptr(args[0].integer);
@@ -1440,9 +1430,17 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
     args = env;
     state = disc->user;
     switch (sym->index) {
-    case A_ARGV:
-      v.string = getArg(args[0].integer, state);
+    case A_ARGV: {
+      strview_t arg;
+      if (getArg(args[0].integer, state, &arg) != 0) {
+        return (Extype_t){0};
+      }
+      v.string = exstralloc(pgm, arg.size + 1);
+      if (arg.size > 0) {
+        memcpy(v.string, arg.data, arg.size);
+      }
       break;
+    }
     default:
       exerror("unknown array name: %s", sym->name);
       v.string = 0;
@@ -1481,9 +1479,12 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
     case V_infname:
       v.string = state->infname;
       break;
-    case V_ARGC:
-      v.integer = state->argc;
+    case V_ARGC: {
+      const size_t size = LIST_SIZE(&state->args);
+      assert(size <= LLONG_MAX);
+      v.integer = (long long)size;
       break;
+    }
     case V_travtype:
       v.integer = state->tvt;
       break;
@@ -1616,7 +1617,9 @@ static Extype_t length(Exid_t *rhs, Exdisc_t *disc) {
   switch (rhs->index) {
   case A_ARGV: {
     Gpr_t *const state = disc->user;
-    v.integer = state->argc;
+    const size_t size = LIST_SIZE(&state->args);
+    assert(size <= LLONG_MAX);
+    v.integer = (long long)size;
     break;
   }
   default:
@@ -1639,7 +1642,8 @@ static int in(Extype_t lhs, Exid_t *rhs, Exdisc_t *disc) {
   switch (rhs->index) {
   case A_ARGV: {
     Gpr_t *const state = disc->user;
-    return lhs.integer >= 0 && lhs.integer < state->argc;
+    return lhs.integer >= 0 &&
+           (unsigned long long)lhs.integer < LIST_SIZE(&state->args);
   }
   default:
     exerror("unknown array name: %s", rhs->name);
@@ -2241,10 +2245,10 @@ static case_stmt *mkStmts(Expr_t *prog, char *src, case_infos_t cases,
                           const char *lbl) {
   agxbuf tmp = {0};
 
-  case_stmt *cs = gv_calloc(case_infos_size(&cases), sizeof(case_stmt));
+  case_stmt *cs = gv_calloc(LIST_SIZE(&cases), sizeof(case_stmt));
 
-  for (size_t i = 0; i < case_infos_size(&cases); i++) {
-    case_info *sp = case_infos_at(&cases, i);
+  for (size_t i = 0; i < LIST_SIZE(&cases); i++) {
+    case_info *sp = LIST_AT(&cases, i);
     if (sp->guard) {
       agxbprint(&tmp, "%s_g%" PRISIZE_T, lbl, i);
       cs[i].guard =
@@ -2296,12 +2300,12 @@ static bool mkBlock(comp_block *bp, Expr_t *prog, char *src, parse_block *inp,
   }
 
   codePhase = 2;
-  if (!case_infos_is_empty(&inp->node_stmts)) {
+  if (!LIST_IS_EMPTY(&inp->node_stmts)) {
     static const char PREFIX[] = "_nd";
     agxbuf label = {0};
     symbols[0].type = T_node;
     tchk[V_this][1] = Y(V);
-    bp->n_nstmts = case_infos_size(&inp->node_stmts);
+    bp->n_nstmts = LIST_SIZE(&inp->node_stmts);
     agxbprint(&label, "%s%" PRISIZE_T, PREFIX, i);
     bp->node_stmts = mkStmts(prog, src, inp->node_stmts, agxbuse(&label));
     agxbfree(&label);
@@ -2311,12 +2315,12 @@ static bool mkBlock(comp_block *bp, Expr_t *prog, char *src, parse_block *inp,
   }
 
   codePhase = 3;
-  if (!case_infos_is_empty(&inp->edge_stmts)) {
+  if (!LIST_IS_EMPTY(&inp->edge_stmts)) {
     static const char PREFIX[] = "_eg";
     agxbuf label = {0};
     symbols[0].type = T_edge;
     tchk[V_this][1] = Y(E);
-    bp->n_estmts = case_infos_size(&inp->edge_stmts);
+    bp->n_estmts = LIST_SIZE(&inp->edge_stmts);
     agxbprint(&label, "%s%" PRISIZE_T, PREFIX, i);
     bp->edge_stmts = mkStmts(prog, src, inp->edge_stmts, agxbuse(&label));
     agxbfree(&label);
@@ -2383,14 +2387,13 @@ comp_prog *compileProg(parse_prog *inp, Gpr_t *state, compflags_t flags) {
       goto finish;
   }
 
-  if (!parse_blocks_is_empty(&inp->blocks)) {
+  if (!LIST_IS_EMPTY(&inp->blocks)) {
     comp_block *bp;
 
-    p->blocks = bp =
-        gv_calloc(parse_blocks_size(&inp->blocks), sizeof(comp_block));
+    p->blocks = bp = gv_calloc(LIST_SIZE(&inp->blocks), sizeof(comp_block));
 
-    for (size_t i = 0; i < parse_blocks_size(&inp->blocks); bp++, i++) {
-      parse_block *ibp = parse_blocks_at(&inp->blocks, i);
+    for (size_t i = 0; i < LIST_SIZE(&inp->blocks); bp++, i++) {
+      parse_block *ibp = LIST_AT(&inp->blocks, i);
       uses_graph |= mkBlock(bp, p->prog, inp->source, ibp, i);
       if (getErrorErrors())
         goto finish;
