@@ -36,29 +36,27 @@
 #include <common/pointset.h>
 #include <util/alloc.h>
 #include <util/exit.h>
+#include <util/gv_math.h>
+#include <util/list.h>
 #include <util/unused.h>
 
 typedef struct {
-    int d;
+    double d;
     Agedge_t* e;
 } epair_t;
 
-#ifdef DEBUG
-#define DEBUG_FN /* nothing */
-#else
-#define DEBUG_FN UNUSED
-#endif
-
-static DEBUG_FN void emitSearchGraph(FILE *fp, sgraph *sg);
-static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
-                               route *route_list, epair_t[]);
+static UNUSED void emitSearchGraph(FILE *fp, sgraph *sg);
+static UNUSED void emitGraph(FILE *fp, maze *mp, size_t n_edges,
+                             route *route_list, epair_t[]);
 #ifdef DEBUG
 int odb_flags;
 #endif
 
 #define CELL(n) ((cell*)ND_alg(n))
-#define MID(a,b) (((a)+(b))/2.0)
-#define SC 1
+
+static double MID(double a, double b) {
+  return (a + b) / 2.0;
+}
 
 /* cellOf:
  * Given 2 snodes sharing a cell, return the cell.
@@ -68,47 +66,28 @@ cellOf (snode* p, snode* q)
 {
     cell* cp = p->cells[0];
     if (cp == q->cells[0] || cp == q->cells[1]) return cp;
-    else return p->cells[1];
+    return p->cells[1];
 }
 
-static pointf
-midPt (cell* cp)
-{
-    pointf p;
-    p.x = MID(cp->bb.LL.x,cp->bb.UR.x);
-    p.y = MID(cp->bb.LL.y,cp->bb.UR.y);
-    return p;
+static pointf midPt(const cell *cp) {
+  return mid_pointf(cp->bb.LL, cp->bb.UR);
 }
 
 /* sidePt:
  * Given a cell and an snode on one of its sides, return the
  * midpoint of the side.
  */
-static pointf
-sidePt (snode* ptr, cell* cp)
-{
-    pointf pt;
-    if (cp == ptr->cells[1]) {
-	if (ptr->isVert) {
-	    pt.x = cp->bb.LL.x;
-	    pt.y = MID(cp->bb.LL.y,cp->bb.UR.y);
-	}
-	else {
-	    pt.x = MID(cp->bb.LL.x,cp->bb.UR.x);
-	    pt.y = cp->bb.LL.y;
+static pointf sidePt(const snode ptr, const cell* cp) {
+    if (cp == ptr.cells[1]) {
+	if (ptr.isVert) {
+	    return (pointf){.x = cp->bb.LL.x, .y = MID(cp->bb.LL.y, cp->bb.UR.y)};
 	}
+	return (pointf){.x = MID(cp->bb.LL.x, cp->bb.UR.x), .y = cp->bb.LL.y};
     }
-    else {
-	if (ptr->isVert) {
-	    pt.x = cp->bb.UR.x;
-	    pt.y = MID(cp->bb.LL.y,cp->bb.UR.y);
-	}
-	else {
-	    pt.x = MID(cp->bb.LL.x,cp->bb.UR.x);
-	    pt.y = cp->bb.UR.y;
-	}
+    if (ptr.isVert) {
+	return (pointf){.x = cp->bb.UR.x, .y = MID(cp->bb.LL.y, cp->bb.UR.y)};
     }
-    return pt;
+    return (pointf){.x = MID(cp->bb.LL.x, cp->bb.UR.x), .y = cp->bb.UR.y};
 }
 
 /* setSet:
@@ -145,23 +124,17 @@ setSeg (segment* sp, bool dir, double fix, double b1, double b2, int l1, int l2)
 static route
 convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 {
-    route rte;
     snode* ptr;
     snode* next;
     snode* prev;  /* node in shortest path just previous to next */
-    size_t sz = 0;
     cell* cp;
     cell* ncp;
     segment seg;
     double fix, b1, b2;
     int l1, l2;
-    pointf bp1, bp2, prevbp = {0.0,0.0};  /* bend points */
+    pointf bp1, prevbp = {0.0,0.0};  /* bend points */
 
-	/* count no. of nodes in shortest path */
-    for (ptr = fst; ptr; ptr = N_DAD(ptr)) sz++;
-    rte.n = 0;
-    assert(sz >= 2);
-    rte.segs = gv_calloc(sz - 2, sizeof(segment));  /* at most sz-2 segments */
+    LIST(segment) rte = {0};
 
     seg.prev = seg.next = 0;
     ptr = prev = N_DAD(fst);
@@ -170,17 +143,14 @@ convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 	cp = ptr->cells[1];
     else
 	cp = ptr->cells[0];
-    bp1 = sidePt (ptr, cp);
+    bp1 = sidePt(*ptr, cp);
     while (N_DAD(next)!=NULL) {
 	ncp = cellOf (prev, next);
 	updateWts (g, ncp, N_EDGE(ptr));
 
         /* add seg if path bends or at end */
 	if (ptr->isVert != next->isVert || N_DAD(next) == lst) {
-	    if (ptr->isVert != next->isVert)
-		bp2 = midPt (ncp);
-	    else
-		bp2 = sidePt(next, ncp);
+	    const pointf bp2 = ptr->isVert != next->isVert ? midPt (ncp) : sidePt(*next, ncp);
 	    if (ptr->isVert) {   /* horizontal segment */
 		if (ptr == N_DAD(fst)) l1 = B_NODE;
 		else if (prevbp.y > bp1.y) l1 = B_UP;
@@ -208,12 +178,11 @@ convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 		b2 = ncp->bb.LL.y;
 	    }
 	    setSeg (&seg, !ptr->isVert, fix, b1, b2, l1, l2);
-	    rte.segs[rte.n++] = seg;
+	    LIST_APPEND(&rte, seg);
 	    cp = ncp;
 	    prevbp = bp1;
 	    bp1 = bp2;
 	    if (ptr->isVert != next->isVert && N_DAD(next) == lst) {
-		bp2 = sidePt(next, ncp);
 		l2 = B_NODE;
 		if (next->isVert) {   /* horizontal segment */
 		    if (prevbp.y > bp1.y) l1 = B_UP;
@@ -230,7 +199,7 @@ convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 		    b2 = ncp->bb.LL.y;
 		}
 		setSeg (&seg, !next->isVert, fix, b1, b2, l1, l2);
-		rte.segs[rte.n++] = seg;
+		LIST_APPEND(&rte, seg);
 	    }
 	    ptr = next;
 	}
@@ -238,15 +207,16 @@ convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 	next = N_DAD(next);
     }
 
-    rte.segs = gv_recalloc(rte.segs, sz - 2, rte.n, sizeof(segment));
-    for (size_t i=0; i<rte.n; i++) {
+    route ret = {0};
+    LIST_DETACH(&rte, &ret.segs, &ret.n);
+    for (size_t i = 0; i < ret.n; i++) {
 	if (i > 0)
-	    rte.segs[i].prev = rte.segs + (i-1);
-	if (i < rte.n-1)
-	    rte.segs[i].next = rte.segs + (i+1);
+	    ret.segs[i].prev = ret.segs + (i - 1);
+	if (i < ret.n - 1)
+	    ret.segs[i].next = ret.segs + (i + 1);
     }
 
-    return rte;
+    return ret;
 }
 
 typedef struct {
@@ -258,7 +228,7 @@ typedef struct {
 static void freeChannel(void *chan) {
     channel *cp = chan;
     free_graph (cp->G);
-    seg_list_free(&cp->seg_list);
+    LIST_FREE(&cp->seg_list);
     free (cp);
 }
 
@@ -282,21 +252,19 @@ static int chancmpid(void *k1, void *k2) {
   const paird *key2 = k2;
   if (key1->p1 > key2->p1) {
     if (key1->p2 <= key2->p2) return 0;
-    else return 1;
+    return 1;
   }
-  else if (key1->p1 < key2->p1) {
+  if (key1->p1 < key2->p1) {
     if (key1->p2 >= key2->p2) return 0;
-    else return -1;
+    return -1;
   }
-  else return 0;
+  return 0;
 }   
 
 static int dcmpid(void *k1, void *k2) {
   const double *key1 = k1;
   const double *key2 = k2;
-  if (*key1 > *key2) return 1;
-  else if (*key1 < *key2) return -1;
-  else return 0;
+  return fcmp(*key1, *key2);
 }   
 
 static Dtdisc_t chanDisc = {
@@ -336,11 +304,10 @@ addChan (Dt_t* chdict, channel* cp, double j)
 static Dt_t*
 extractHChans (maze* mp)
 {
-    int i;
     snode* np;
     Dt_t* hchans = dtopen (&chanItemDisc, Dtoset);
 
-    for (i = 0; i < mp->ncells; i++) {
+    for (size_t i = 0; i < mp->ncells; i++) {
 	channel* chp;
 	cell* cp = mp->cells+i;
 	cell* nextcp;
@@ -373,11 +340,10 @@ extractHChans (maze* mp)
 static Dt_t*
 extractVChans (maze* mp)
 {
-    int i;
     snode* np;
     Dt_t* vchans = dtopen (&chanItemDisc, Dtoset);
 
-    for (i = 0; i < mp->ncells; i++) {
+    for (size_t i = 0; i < mp->ncells; i++) {
 	channel* chp;
 	cell* cp = mp->cells+i;
 	cell* nextcp;
@@ -410,8 +376,8 @@ extractVChans (maze* mp)
 static void
 insertChan (channel* chan, segment* seg)
 {
-    seg->ind_no = seg_list_size(&chan->seg_list);
-    seg_list_append(&chan->seg_list, seg);
+    seg->ind_no = LIST_SIZE(&chan->seg_list);
+    LIST_APPEND(&chan->seg_list, seg);
 }
 
 static channel*
@@ -450,19 +416,11 @@ assignSegs (size_t nrtes, route* route_list, maze* mp)
 static void
 addLoop (sgraph* sg, cell* cp, snode* dp, snode* sp)
 {
-    int i;
-    int onTop;
-
-    for (i = 0; i < cp->nsides; i++) {
+    for (size_t i = 0; i < cp->nsides; i++) {
 	snode* onp = cp->sides[i];
 
 	if (onp->isVert) continue;
-	if (onp->cells[0] == cp) {
-	    onTop = 1;
-	}
-	else {
-	    onTop = 0;
-	}
+	const bool onTop = onp->cells[0] == cp;
 	if (onTop)
 	    createSEdge (sg, sp, onp, 0);  /* FIX weight */
 	else
@@ -478,9 +436,7 @@ addLoop (sgraph* sg, cell* cp, snode* dp, snode* sp)
 static void
 addNodeEdges (sgraph* sg, cell* cp, snode* np)
 {
-    int i;
-
-    for (i = 0; i < cp->nsides; i++) {
+    for (size_t i = 0; i < cp->nsides; i++) {
 	snode* onp = cp->sides[i];
 
 	createSEdge (sg, np, onp, 0);  /* FIX weight */
@@ -525,17 +481,17 @@ static void putSeg (FILE* fp, segment* seg)
       seg->p.p2, seg->comm_coord, bendToStr (seg->l1), bendToStr (seg->l2));
 }
 
-static DEBUG_FN void dumpChanG(channel *cp, double v) {
-  if (seg_list_size(&cp->seg_list) < 2) return;
+static UNUSED void dumpChanG(channel *cp, double v) {
+  if (LIST_SIZE(&cp->seg_list) < 2) return;
   fprintf (stderr, "channel %.0f (%f,%f)\n", v, cp->p.p1, cp->p.p2);
-  for (size_t k = 0; k < seg_list_size(&cp->seg_list); ++k) {
+  for (size_t k = 0; k < LIST_SIZE(&cp->seg_list); ++k) {
     const adj_list_t adj = cp->G->vertices[k].adj_list;
-    if (adj_list_is_empty(&adj)) continue;
-    putSeg(stderr, seg_list_get(&cp->seg_list, k));
+    if (LIST_IS_EMPTY(&adj)) continue;
+    putSeg(stderr, LIST_GET(&cp->seg_list, k));
     fputs (" ->\n", stderr);
-    for (size_t i = 0; i < adj_list_size(&adj); ++i) {
+    for (size_t i = 0; i < LIST_SIZE(&adj); ++i) {
       fputs ("     ", stderr);
-      putSeg(stderr, seg_list_get(&cp->seg_list, adj_list_get(&adj, i)));
+      putSeg(stderr, LIST_GET(&cp->seg_list, LIST_GET(&adj, i)));
       fputs ("\n", stderr);
     }
   }
@@ -553,13 +509,13 @@ assignTrackNo (Dt_t* chans)
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    if (!seg_list_is_empty(&cp->seg_list)) {
+	    if (!LIST_IS_EMPTY(&cp->seg_list)) {
 #ifdef DEBUG
     if (odb_flags & ODB_CHANG) dumpChanG (cp, ((chanItem*)l1)->v);
 #endif
 		top_sort (cp->G);
-		for (size_t k = 0; k < seg_list_size(&cp->seg_list); ++k)
-		    seg_list_get(&cp->seg_list, k)->track_no = cp->G->vertices[k].topsort_order+1;
+		for (size_t k = 0; k < LIST_SIZE(&cp->seg_list); ++k)
+		    LIST_GET(&cp->seg_list, k)->track_no = cp->G->vertices[k].topsort_order+1;
 	    }
    	}
     }
@@ -577,7 +533,7 @@ create_graphs(Dt_t* chans)
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    cp->G = make_graph(seg_list_size(&cp->seg_list));
+	    cp->G = make_graph(LIST_SIZE(&cp->seg_list));
    	}
     }
 }
@@ -595,14 +551,14 @@ static int
 overlapSeg (segment* S1, segment* S2, bend T1, bend T2)
 {
 	if(S1->p.p2<S2->p.p2) {
-		if(S1->l2==T1&&S2->l1==T2) return(-1);
-		else if(S1->l2==T2&&S2->l1==T1) return(1);
-		else return(0);
+		if (S1->l2 == T1 && S2->l1 == T2) return -1;
+		if (S1->l2 == T2 && S2->l1 == T1) return 1;
+		return 0;
 	}
 	if (S1->p.p2 > S2->p.p2) {
-		if(S2->l1==T2&&S2->l2==T2) return(-1);
-		else if (S2->l1==T1&&S2->l2==T1) return(1);
-		else return(0);
+		if (S2->l1 == T2 && S2->l2 == T2) return -1;
+		if (S2->l1 == T1 && S2->l2 == T1) return 1;
+		return 0;
 	}
 	if (S2->l1 == T2) return eqEndSeg (S1->l2, S2->l2, T1, T2);
 	return -1 * eqEndSeg(S2->l2, S1->l2, T1, T2);
@@ -613,9 +569,9 @@ ellSeg (bend S1l1, bend S1l2, bend T)
 {
     if (S1l1 == T) {
 	if (S1l2== T) return -1;
-	else return 0;
+	return 0;
     }
-    else return 1;
+    return 1;
 }
 
 static int
@@ -630,47 +586,46 @@ segCmp (segment* S1, segment* S2, bend T1, bend T2)
     if (S2->p.p1 < S1->p.p1 && S1->p.p1 < S2->p.p2)
 	return -1*overlapSeg (S2, S1, T1, T2);
     if (S1->p.p1 == S2->p.p1) {
-	if(S1->p.p2==S2->p.p2) {
-	    if (S1->l1 == S2->l1 && S1->l2 == S2->l2)
-		return 0;
-	    if (S2->l1 == S2->l2) {
-		if (S2->l1 == T1) return 1;
-		if (S2->l1 == T2) return -1;
-		if (S1->l1 != T1 && S1->l2 != T1) return 1;
-		if (S1->l1 != T2 && S1->l2 != T2) return -1;
-		return 0;
-	    }
-	    if (S2->l1 == T1 && S2->l2 == T2) {
-		if (S1->l1 != T1 && S1->l2 == T2) return 1;
-		if (S1->l1 == T1 && S1->l2 != T2) return -1;
-		return 0;
-	    }
-	    if (S2->l2 == T1 && S2->l1 == T2) {
-		if (S1->l2 != T1 && S1->l1 == T2) return 1;
-		if (S1->l2 == T1 && S1->l1 != T2) return -1;
-		return 0;
-	    }
-	    if (S2->l1 == B_NODE && S2->l2 == T1) {
-		return ellSeg (S1->l1, S1->l2, T1);
-	    }
-	    if (S2->l1 == B_NODE && S2->l2 == T2) {
-		return -1*ellSeg (S1->l1, S1->l2, T2);
-	    }
-	    if (S2->l1 == T1 && S2->l2 == B_NODE) {
-		return ellSeg (S1->l2, S1->l1, T1);
-	    }
-	    /* ((S2->l1==T2)&&(S2->l2==B_NODE)) */
-	    return -1 * ellSeg(S1->l2, S1->l1, T2);
-	}
 	if (S1->p.p2 < S2->p.p2) {
 	    if(S1->l2==T1)
 		return eqEndSeg (S2->l1, S1->l1, T1, T2);
 	    return -1 * eqEndSeg(S2->l1, S1->l1, T1, T2);
 	}
-	/* S1->p.p2>S2->p.p2 */
-	if (S2->l2 == T2)
-	    return eqEndSeg(S1->l1, S2->l1, T1, T2);
-	return -1 * eqEndSeg(S1->l1, S2->l1, T1, T2);
+	if (S1->p.p2 > S2->p.p2) {
+	    if (S2->l2 == T2)
+		return eqEndSeg(S1->l1, S2->l1, T1, T2);
+	    return -1 * eqEndSeg(S1->l1, S2->l1, T1, T2);
+	}
+	if (S1->l1 == S2->l1 && S1->l2 == S2->l2)
+	    return 0;
+	if (S2->l1 == S2->l2) {
+	    if (S2->l1 == T1) return 1;
+	    if (S2->l1 == T2) return -1;
+	    if (S1->l1 != T1 && S1->l2 != T1) return 1;
+	    if (S1->l1 != T2 && S1->l2 != T2) return -1;
+	    return 0;
+	}
+	if (S2->l1 == T1 && S2->l2 == T2) {
+	    if (S1->l1 != T1 && S1->l2 == T2) return 1;
+	    if (S1->l1 == T1 && S1->l2 != T2) return -1;
+	    return 0;
+	}
+	if (S2->l2 == T1 && S2->l1 == T2) {
+	    if (S1->l2 != T1 && S1->l1 == T2) return 1;
+	    if (S1->l2 == T1 && S1->l1 != T2) return -1;
+	    return 0;
+	}
+	if (S2->l1 == B_NODE && S2->l2 == T1) {
+	    return ellSeg (S1->l1, S1->l2, T1);
+	}
+	if (S2->l1 == B_NODE && S2->l2 == T2) {
+	    return -1*ellSeg (S1->l1, S1->l2, T2);
+	}
+	if (S2->l1 == T1 && S2->l2 == B_NODE) {
+	    return ellSeg (S1->l2, S1->l1, T1);
+	}
+	/* ((S2->l1==T2)&&(S2->l2==B_NODE)) */
+	return -1 * ellSeg(S1->l2, S1->l1, T2);
     }
     if (S1->p.p2 == S2->p.p1) {
 	if (S1->l2 == S2->l1) return 0;
@@ -715,13 +670,12 @@ static int
 add_edges_in_G(channel* cp)
 {
     seg_list_t *seg_list = &cp->seg_list;
-    const size_t size = seg_list_size(&cp->seg_list);
+    const size_t size = LIST_SIZE(&cp->seg_list);
     rawgraph* G = cp->G;
 
     for (size_t x = 0; x + 1 < size; ++x) {
 	for (size_t y = x + 1; y < size; ++y) {
-	    const int cmp = seg_cmp(seg_list_get(seg_list, x),
-	                            seg_list_get(seg_list, y));
+	    const int cmp = seg_cmp(LIST_GET(seg_list, x), LIST_GET(seg_list, y));
 	    if (cmp == -2) {
 		return -1;
 	    } else if (cmp > 0) {
@@ -738,16 +692,11 @@ add_edges_in_G(channel* cp)
 static int
 add_np_edges (Dt_t* chans)
 {
-    Dt_t* lp;
-    Dtlink_t* l1;
-    Dtlink_t* l2;
-    channel* cp;
-
-    for (l1 = dtflatten (chans); l1; l1 = dtlink(chans,l1)) {
-	lp = ((chanItem*)l1)->chans;
-	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
-	    cp = (channel*)l2;
-	    if (!seg_list_is_empty(&cp->seg_list))
+    for (Dtlink_t *l1 = dtflatten(chans); l1; l1 = dtlink(chans, l1)) {
+	Dt_t *const lp = ((chanItem*)l1)->chans;
+	for (Dtlink_t *l2 = dtflatten(lp); l2; l2 = dtlink(lp, l2)) {
+	    channel *const cp = (channel*)l2;
+	    if (!LIST_IS_EMPTY(&cp->seg_list))
 		if (add_edges_in_G(cp)) {
 		  return -1;
 		}
@@ -977,35 +926,35 @@ addPEdges (channel* cp, maze* mp)
     rawgraph* G = cp->G;
     seg_list_t *segs = &cp->seg_list;
 
-    for(size_t i = 0; i + 1 < seg_list_size(&cp->seg_list); ++i) {
-	for(size_t j = i + 1; j < seg_list_size(&cp->seg_list); ++j) {
+    for(size_t i = 0; i + 1 < LIST_SIZE(&cp->seg_list); ++i) {
+	for(size_t j = i + 1; j < LIST_SIZE(&cp->seg_list); ++j) {
 	    if (!edge_exists(G,i,j) && !edge_exists(G,j,i)) {
-		if (is_parallel(seg_list_get(segs, i), seg_list_get(segs, j))) {
+		if (is_parallel(LIST_GET(segs, i), LIST_GET(segs, j))) {
 		/* get_directions */
-		    if (seg_list_get(segs, i)->prev == 0) {
-			if (seg_list_get(segs, j)->prev == 0)
+		    if (LIST_GET(segs, i)->prev == 0) {
+			if (LIST_GET(segs, j)->prev == 0)
 			    dir = 0;
 			else
 			    dir = 1;
 		    }
-		    else if (seg_list_get(segs, j)->prev == 0) {
+		    else if (LIST_GET(segs, j)->prev == 0) {
 			dir = 1;
 		    }
 		    else {
-			if (seg_list_get(segs, i)->prev->comm_coord ==
-			    seg_list_get(segs, j)->prev->comm_coord)
+			if (LIST_GET(segs, i)->prev->comm_coord ==
+			    LIST_GET(segs, j)->prev->comm_coord)
 			    dir = 0;
 			else
 			    dir = 1;
 		    }
 
-		    if (decide_point(&p, seg_list_get(segs, i), seg_list_get(segs, j), 0, dir)
+		    if (decide_point(&p, LIST_GET(segs, i), LIST_GET(segs, j), 0, dir)
 		        != 0) {
 			return -1;
 		    }
 		    hops.a = p.a;
 		    prec1 = p.b;
-		    if (decide_point(&p, seg_list_get(segs, i), seg_list_get(segs, j), 1,
+		    if (decide_point(&p, LIST_GET(segs, i), LIST_GET(segs, j), 1,
 		                     1 - dir) != 0) {
 			return -1;
 		    }
@@ -1013,36 +962,36 @@ addPEdges (channel* cp, maze* mp)
 		    prec2 = p.b;
 
 		    if (prec1 == -1) {
-			set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), dir, 0,
+			set_parallel_edges(LIST_GET(segs, j), LIST_GET(segs, i), dir, 0,
 			                   hops.a, mp);
-			set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), 1 - dir, 1,
+			set_parallel_edges(LIST_GET(segs, j), LIST_GET(segs, i), 1 - dir, 1,
 			                   hops.b, mp);
 			if(prec2==1)
-			    removeEdge(seg_list_get(segs, i), seg_list_get(segs, j), 1 - dir, mp);
+			    removeEdge(LIST_GET(segs, i), LIST_GET(segs, j), 1 - dir, mp);
 		    } else if (prec1 == 0) {
 			if (prec2 == -1) {
-			    set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), dir, 0,
+			    set_parallel_edges(LIST_GET(segs, j), LIST_GET(segs, i), dir, 0,
 			                       hops.a, mp);
-			    set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), 1 - dir,
+			    set_parallel_edges(LIST_GET(segs, j), LIST_GET(segs, i), 1 - dir,
 			                       1, hops.b, mp);
 			} else if (prec2 == 0) {
-			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			    set_parallel_edges(LIST_GET(segs, i), LIST_GET(segs, j), 0, dir,
 			                       hops.a, mp);
-			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1,
+			    set_parallel_edges(LIST_GET(segs, i), LIST_GET(segs, j), 1,
 			                       1 - dir, hops.b, mp);
 			} else if (prec2 == 1) {
-			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			    set_parallel_edges(LIST_GET(segs, i), LIST_GET(segs, j), 0, dir,
 			                       hops.a, mp);
-			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1,
+			    set_parallel_edges(LIST_GET(segs, i), LIST_GET(segs, j), 1,
 			                       1 - dir, hops.b, mp);
 			}
 		    } else if (prec1 == 1) {
-			set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			set_parallel_edges(LIST_GET(segs, i), LIST_GET(segs, j), 0, dir,
 			                   hops.a, mp);
-			set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1, 1 - dir,
+			set_parallel_edges(LIST_GET(segs, i), LIST_GET(segs, j), 1, 1 - dir,
 			                   hops.b, mp);
 			if(prec2==-1)
-			    removeEdge(seg_list_get(segs, i), seg_list_get(segs, j), 1 - dir, mp);
+			    removeEdge(LIST_GET(segs, i), LIST_GET(segs, j), 1 - dir, mp);
 		    }
 		}
 	    }
@@ -1055,13 +1004,9 @@ addPEdges (channel* cp, maze* mp)
 static int
 add_p_edges (Dt_t* chans, maze* mp)
 {
-    Dt_t* lp;
-    Dtlink_t* l1;
-    Dtlink_t* l2;
-
-    for (l1 = dtflatten (chans); l1; l1 = dtlink(chans,l1)) {
-	lp = ((chanItem*)l1)->chans;
-	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
+    for (Dtlink_t *l1 = dtflatten(chans); l1; l1 = dtlink(chans, l1)) {
+	Dt_t *const lp = ((chanItem*)l1)->chans;
+	for (Dtlink_t *l2 = dtflatten(lp); l2; l2 = dtlink(lp, l2)) {
 	    if (addPEdges((channel*)l2, mp) != 0) {
 	        return -1;
 	    }
@@ -1105,66 +1050,46 @@ static double
 vtrack (segment* seg, maze* m)
 {
   channel* chp = chanSearch(m->vchans, seg);
-  const double f = (double)seg->track_no / ((double)seg_list_size(&chp->seg_list) + 1);
-  double left = chp->cp->bb.LL.x;
-  double right = chp->cp->bb.UR.x;
-  return left + f*(right-left);
+  const double f = seg->track_no / ((double)LIST_SIZE(&chp->seg_list) + 1);
+  const pointf interp = interpolate_pointf(f, chp->cp->bb.LL, chp->cp->bb.UR);
+  return interp.x;
 }
 
 static double htrack(segment *seg, maze *m) {
   channel* chp = chanSearch(m->hchans, seg);
-  double f = 1.0 - (double)seg->track_no / ((double)seg_list_size(&chp->seg_list) + 1);
+  double f = 1.0 - seg->track_no / ((double)LIST_SIZE(&chp->seg_list) + 1);
   double lo = chp->cp->bb.LL.y;
   double hi = chp->cp->bb.UR.y;
   return round(lo + f * (hi - lo));
 }
 
-static pointf
-addPoints(pointf p0, pointf p1)
-{
-    p0.x += p1.x;
-    p0.y += p1.y;
-    return p0;
-}
-
 static void attachOrthoEdges(maze *mp, size_t n_edges, route* route_list,
                              splineInfo *sinfo, epair_t es[], bool doLbls) {
-    int ipt;
-    pointf* ispline = 0;
-    size_t splsz = 0;
-    pointf p, p1, q1;
-    route rte;
-    segment* seg;
-    Agedge_t* e;
+    LIST(pointf) ispline = {0};
     textlabel_t* lbl;
 
     for (size_t irte = 0; irte < n_edges; irte++) {
-	e = es[irte].e;
-	p1 = addPoints(ND_coord(agtail(e)), ED_tail_port(e).p);
-	q1 = addPoints(ND_coord(aghead(e)), ED_head_port(e).p);
+	Agedge_t *const e = es[irte].e;
+	const pointf p1 = add_pointf(ND_coord(agtail(e)), ED_tail_port(e).p);
+	const pointf q1 = add_pointf(ND_coord(aghead(e)), ED_head_port(e).p);
 
-	rte = route_list[irte];
+	route rte = route_list[irte];
 	size_t npts = 1 + 3*rte.n;
-	if (npts > splsz) {
-		free (ispline);
-		ispline = gv_calloc(npts, sizeof(pointf));
-		splsz = npts;
-	}
+	LIST_RESERVE(&ispline, npts);
 	    
-	seg = rte.segs;
+	segment *seg = rte.segs;
 	if (seg == NULL) {
 		continue;
 	}
+	pointf p;
 	if (seg->isVert) {
-		p.x = vtrack(seg, mp);
-		p.y = p1.y;
+		p = (pointf){.x = vtrack(seg, mp), .y = p1.y};
 	}
 	else {
-		p.y = htrack(seg, mp);
-		p.x = p1.x;
+		p = (pointf){.x = p1.x, .y = htrack(seg, mp)};
 	}
-	ispline[0] = ispline[1] = p;
-	ipt = 2;
+	LIST_APPEND(&ispline, p);
+	LIST_APPEND(&ispline, p);
 
 	for (size_t i = 1;i<rte.n;i++) {
 		seg = rte.segs+i;
@@ -1172,34 +1097,34 @@ static void attachOrthoEdges(maze *mp, size_t n_edges, route* route_list,
 		    p.x = vtrack(seg, mp);
 		else
 		    p.y = htrack(seg, mp);
-		ispline[ipt+2] = ispline[ipt+1] = ispline[ipt] = p;
-		ipt += 3;
+		LIST_APPEND(&ispline, p);
+		LIST_APPEND(&ispline, p);
+		LIST_APPEND(&ispline, p);
 	}
 
 	if (seg->isVert) {
-		p.x = vtrack(seg, mp);
-		p.y = q1.y;
+		p = (pointf){.x = vtrack(seg, mp), .y = q1.y};
 	}
 	else {
-		p.y = htrack(seg, mp);
-		p.x = q1.x;
+		p = (pointf){.x = q1.x, .y = htrack(seg, mp)};
 	}
-	ispline[ipt] = ispline[ipt+1] = p;
+	LIST_APPEND(&ispline, p);
+	LIST_APPEND(&ispline, p);
 	if (Verbose > 1)
 	    fprintf(stderr, "ortho %s %s\n", agnameof(agtail(e)),agnameof(aghead(e)));
-	clip_and_install(e, aghead(e), ispline, npts, sinfo);
+	clip_and_install(e, aghead(e), LIST_FRONT(&ispline), LIST_SIZE(&ispline),
+	                 sinfo);
 	if (doLbls && (lbl = ED_label(e)) && !lbl->set)
 	    addEdgeLabels(e);
+	LIST_CLEAR(&ispline);
     }
-    free(ispline);
+    LIST_FREE(&ispline);
 }
 
-static int
-edgeLen (Agedge_t* e)
-{
+static double edgeLen(Agedge_t *e) {
     pointf p = ND_coord(agtail(e));
     pointf q = ND_coord(aghead(e));
-    return (int)DIST2(p,q);
+    return DIST2(p, q);
 }
 
 static int edgecmp(const void *x, const void *y) {
@@ -1226,25 +1151,13 @@ static bool swap_ends_p(edge_t * e)
     return false;
 }
 
-static splineInfo sinfo = { swap_ends_p, spline_merge, true, true };
-
 /* orthoEdges:
  * For edges without position information, construct an orthogonal routing.
  * If useLbls is true, use edge label info when available to guide routing, 
  * and set label pos for those edges for which this info is not available.
  */
 void orthoEdges(Agraph_t *g, bool useLbls) {
-    sgraph* sg;
-    maze* mp;
-    route* route_list;
-    int gstart;
-    Agnode_t* n;
-    Agedge_t* e;
-    snode* sn;
-    snode* dn;
     epair_t* es = gv_calloc(agnedges(g), sizeof(epair_t));
-    cell* start;
-    cell* dest;
     PointSet* ps = NULL;
     textlabel_t* lbl;
 
@@ -1285,27 +1198,27 @@ void orthoEdges(Agraph_t *g, bool useLbls) {
 	agwarningf("Orthogonal edges do not currently handle edge labels. Try using xlabels.\n");
 	useLbls = false;
     }
-    mp = mkMaze(g);
-    sg = mp->sg;
+    maze *const mp = mkMaze(g);
+    sgraph *const sg = mp->sg;
 #ifdef DEBUG
     if (odb_flags & ODB_SGRAPH) emitSearchGraph (stderr, sg);
 #endif
 
     /* store edges to be routed in es, along with their lengths */
     size_t n_edges = 0;
-    for (n = agfstnode (g); n; n = agnxtnode(g, n)) {
-        for (e = agfstout(g, n); e; e = agnxtout(g,e)) {
+    for (Agnode_t *n = agfstnode (g); n; n = agnxtnode(g, n)) {
+        for (Agedge_t *e = agfstout(g, n); e; e = agnxtout(g,e)) {
 	    if (Nop == 2 && ED_spl(e)) continue;
 	    if (Concentrate) {
 		int ti = AGSEQ(agtail(e));
 		int hi = AGSEQ(aghead(e));
 		if (ti <= hi) {
 		    if (isInPS (ps,ti,hi)) continue;
-		    else addPS (ps,ti,hi);
+		    addPS(ps,ti,hi);
 		}
 		else {
 		    if (isInPS (ps,hi,ti)) continue;
-		    else addPS (ps,hi,ti);
+		    addPS(ps,hi,ti);
 		}
 	    }
 	    es[n_edges].e = e;
@@ -1314,21 +1227,21 @@ void orthoEdges(Agraph_t *g, bool useLbls) {
 	}
     }
 
-    route_list = gv_calloc(n_edges, sizeof(route));
+    route *const route_list = gv_calloc(n_edges, sizeof(route));
 
     qsort(es, n_edges, sizeof(epair_t), edgecmp);
 
-    gstart = sg->nnodes;
-    PQgen (sg->nnodes+2);
-    sn = &sg->nodes[gstart];
-    dn = &sg->nodes[gstart+1];
+    const int gstart = sg->nnodes;
+    pq_t *const pq = PQgen(sg->nnodes + 2);
+    snode *const sn = &sg->nodes[gstart];
+    snode *const dn = &sg->nodes[gstart+1];
     for (size_t i = 0; i < n_edges; i++) {
 #ifdef DEBUG
 	if (i > 0 && (odb_flags & ODB_IGRAPH)) emitSearchGraph (stderr, sg);
 #endif
-	e = es[i].e;
-        start = CELL(agtail(e));
-        dest = CELL(aghead(e));
+	Agedge_t *const e = es[i].e;
+        cell *const start = CELL(agtail(e));
+        cell *const dest = CELL(aghead(e));
 
 	if (useLbls && (lbl = ED_label(e)) && lbl->set) {
 	}
@@ -1339,13 +1252,16 @@ void orthoEdges(Agraph_t *g, bool useLbls) {
        		addNodeEdges (sg, dest, dn);
 		addNodeEdges (sg, start, sn);
 	    }
-       	    if (shortPath (sg, dn, sn)) goto orthofinish;
+       	    if (shortPath(pq, sg, dn, sn)) {
+		PQfree(pq);
+		goto orthofinish;
+       	    }
 	}
 	    
        	route_list[i] = convertSPtoRoute(sg, sn, dn);
        	reset (sg);
     }
-    PQfree ();
+    PQfree(pq);
 
     mp->hchans = extractHChans (mp);
     mp->vchans = extractVChans (mp);
@@ -1355,6 +1271,7 @@ void orthoEdges(Agraph_t *g, bool useLbls) {
 #ifdef DEBUG
     if (odb_flags & ODB_ROUTE) emitGraph (stderr, mp, n_edges, route_list, es);
 #endif
+    splineInfo sinfo = {swap_ends_p, spline_merge, true, true};
     attachOrthoEdges(mp, n_edges, route_list, &sinfo, es, useLbls);
 
 orthofinish:
@@ -1371,7 +1288,7 @@ orthofinish:
 #include <common/arith.h>
 #define TRANS 10
 
-static char* prolog2 =
+static const char prolog2[] =
 "%%!PS-Adobe-2.0\n\
 %%%%BoundingBox: (atend)\n\
 /point {\n\
@@ -1403,34 +1320,21 @@ static char* prolog2 =
 } def\n\
 \n";
 
-static char* epilog2 =
-"showpage\n\
-%%%%Trailer\n\
-%%%%BoundingBox: %.f %.f %.f %.f\n";
-
 static pointf coordOf(cell *cp, snode *np) {
-    pointf p;
     if (cp->sides[M_TOP] == np) {
-	p.x = (cp->bb.LL.x + cp->bb.UR.x)/2;
-	p.y = cp->bb.UR.y;
+	return (pointf){.x = (cp->bb.LL.x + cp->bb.UR.x) / 2, .y = cp->bb.UR.y};
     }
-    else if (cp->sides[M_BOTTOM] == np) {
-	p.x = (cp->bb.LL.x + cp->bb.UR.x)/2;
-	p.y = cp->bb.LL.y;
+    if (cp->sides[M_BOTTOM] == np) {
+	return (pointf){.x = (cp->bb.LL.x + cp->bb.UR.x) / 2, .y = cp->bb.LL.y};
     }
-    else if (cp->sides[M_LEFT] == np) {
-	p.y = (cp->bb.LL.y + cp->bb.UR.y)/2;
-	p.x = cp->bb.LL.x;
+    if (cp->sides[M_LEFT] == np) {
+	return (pointf){.x = cp->bb.LL.x, .y = (cp->bb.LL.y + cp->bb.UR.y) / 2};
     }
-    else if (cp->sides[M_RIGHT] == np) {
-	p.y = (cp->bb.LL.y + cp->bb.UR.y)/2;
-	p.x = cp->bb.UR.x;
+    if (cp->sides[M_RIGHT] == np) {
+	return (pointf){.x = cp->bb.UR.x, .y = (cp->bb.LL.y + cp->bb.UR.y) / 2};
     }
-    else {
-	agerrorf("Node not adjacent to cell -- Aborting\n");
-	graphviz_exit(EXIT_FAILURE);
-    }
-    return p;
+    agerrorf("Node not adjacent to cell -- Aborting\n");
+    graphviz_exit(EXIT_FAILURE);
 }
 
 static boxf
@@ -1447,11 +1351,11 @@ emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
 	y = htrack(seg, m);
 	x = (n.UR.x + n.LL.x)/2;
     }
-    bb.LL.x = fmin(bb.LL.x, SC * x);
-    bb.LL.y = fmin(bb.LL.y, SC * y);
-    bb.UR.x = fmax(bb.UR.x, SC * x);
-    bb.UR.y = fmax(bb.UR.y, SC * y);
-    fprintf(fp, "newpath %.0f %.0f moveto\n", SC * x, SC * y);
+    bb.LL.x = fmin(bb.LL.x, x);
+    bb.LL.y = fmin(bb.LL.y, y);
+    bb.UR.x = fmax(bb.UR.x, x);
+    bb.UR.y = fmax(bb.UR.y, y);
+    fprintf(fp, "newpath %.0f %.0f moveto\n", x, y);
 
     for (size_t i = 1;i<rte.n;i++) {
 	seg = rte.segs+i;
@@ -1461,11 +1365,11 @@ emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
 	else {
 	    y = htrack(seg, m);
 	}
-	bb.LL.x = fmin(bb.LL.x, SC * x);
-	bb.LL.y = fmin(bb.LL.y, SC * y);
-	bb.UR.x = fmax(bb.UR.x, SC * x);
-	bb.UR.y = fmax(bb.UR.y, SC * y);
-	fprintf(fp, "%.0f %.0f lineto\n", SC * x, SC * y);
+	bb.LL.x = fmin(bb.LL.x, x);
+	bb.LL.y = fmin(bb.LL.y, y);
+	bb.UR.x = fmax(bb.UR.x, x);
+	bb.UR.y = fmax(bb.UR.y, y);
+	fprintf(fp, "%.0f %.0f lineto\n", x, y);
     }
 
     n = CELL(aghead(e))->bb;
@@ -1477,11 +1381,11 @@ emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
 	y = htrack(seg, m);
 	x = (n.LL.x + n.UR.x)/2;
     }
-    bb.LL.x = fmin(bb.LL.x, SC * x);
-    bb.LL.y = fmin(bb.LL.y, SC * y);
-    bb.UR.x = fmax(bb.UR.x, SC * x);
-    bb.UR.y = fmax(bb.UR.y, SC * y);
-    fprintf(fp, "%.0f %.0f lineto stroke\n", SC * x, SC * y);
+    bb.LL.x = fmin(bb.LL.x, x);
+    bb.LL.y = fmin(bb.LL.y, y);
+    bb.UR.x = fmax(bb.UR.x, x);
+    bb.UR.y = fmax(bb.UR.y, y);
+    fprintf(fp, "%.0f %.0f lineto stroke\n", x, y);
 
     return bb;
 }
@@ -1496,18 +1400,14 @@ emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
  * specified graph layout engine.
  */
 
-static DEBUG_FN void emitSearchGraph(FILE *fp, sgraph *sg) {
-    cell* cp;
-    snode* np;
-    sedge* ep;
+static UNUSED void emitSearchGraph(FILE *fp, sgraph *sg) {
     pointf p;
-    int i;
     fputs ("graph G {\n", fp);
     fputs (" node[shape=point]\n", fp);
     fputs (" layout=neato\n", fp);
-    for (i = 0; i < sg->nnodes; i++) {
-	np = sg->nodes+i;
-	cp = np->cells[0];
+    for (int i = 0; i < sg->nnodes; i++) {
+	snode *const np = sg->nodes+i;
+	cell *cp = np->cells[0];
 	if (cp == np->cells[1]) {
 	    p = midPt(cp);
 	}
@@ -1517,19 +1417,19 @@ static DEBUG_FN void emitSearchGraph(FILE *fp, sgraph *sg) {
 	}
 	fprintf (fp, "  %d [pos=\"%.0f,%.0f!\"]\n", i, p.x, p.y);
     }
-    for (i = 0; i < sg->nedges; i++) {
-	ep = sg->edges+i;
+    for (int i = 0; i < sg->nedges; i++) {
+	sedge *const ep = sg->edges+i;
 	fprintf (fp, "  %d -- %d[label=\"%f\"]\n", ep->v1, ep->v2, ep->weight);
     }
     fputs ("}\n", fp);
 }
 
-static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
-                               route *route_list, epair_t es[]) {
+static UNUSED void emitGraph(FILE *fp, maze *mp, size_t n_edges,
+                             route *route_list, epair_t es[]) {
     boxf absbb = {.LL = {.x = DBL_MAX, .y = DBL_MAX},
                   .UR = {.x = -DBL_MAX, .y = -DBL_MAX}};
 
-    fprintf (fp, "%s", prolog2);
+    fputs(prolog2, fp);
     fprintf (fp, "%d %d translate\n", TRANS, TRANS);
 
     fputs ("0 0 1 setrgbcolor\n", fp);
@@ -1543,7 +1443,7 @@ static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
     }
     
     fputs ("0.8 0.8 0.8 setrgbcolor\n", fp);
-    for (int i = 0; i < mp->ncells; i++) {
+    for (size_t i = 0; i < mp->ncells; i++) {
       const boxf bb = mp->cells[i].bb;
       fprintf (fp, "%f %f %f %f cell\n", bb.LL.x, bb.LL.y, bb.UR.x, bb.UR.y);
       absbb.LL.x = fmin(absbb.LL.x, bb.LL.x);
@@ -1557,5 +1457,6 @@ static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
              .y = absbb.LL.y + TRANS},
       .UR = {.x = absbb.UR.x + TRANS,
              .y = absbb.UR.y + TRANS}};
-    fprintf (fp, epilog2, bbox.LL.x, bbox.LL.y,  bbox.UR.x, bbox.UR.y);
+    fprintf(fp, "showpage\n%%%%Trailer\n%%%%BoundingBox: %.f %.f %.f %.f\n",
+            bbox.LL.x, bbox.LL.y,  bbox.UR.x, bbox.UR.y);
 }
