@@ -285,8 +285,8 @@ static_assert(
 /// @param src List to copy
 #define LIST_COPY(dst, src)                                                    \
   do {                                                                         \
-    (void)((dst)->base == (src)->base);                                        \
     memset((dst), 0, sizeof(*(dst)));                                          \
+    (void)((dst)->base == (src)->base);                                        \
     (dst)->impl = gv_list_copy_((src)->impl, sizeof((src)->base[0]));          \
     (dst)->dtor = (src)->dtor;                                                 \
   } while (0)
@@ -331,10 +331,10 @@ static_assert(
 /// You can think of this macro as having the C type:
 ///
 ///   void LIST_SORT(LIST(<type>) *list,
-///                  int (*comparator)(const void *a, const void *b));
+///                  int (*cmp)(const void *a, const void *b));
 ///
 /// @param list List to operate on
-/// @param comparator How to compare two list items
+/// @param cmp How to compare two list items
 #define LIST_SORT(list, cmp)                                                   \
   gv_list_sort_(&(list)->impl, (cmp), sizeof((list)->base[0]))
 
@@ -441,7 +441,7 @@ static_assert(
 /// @param [out] datap The list data on completion
 /// @param [out] sizep The list size on completion
 #define LIST_DETACH(list, datap, sizep)                                        \
-  gv_list_detach_(&(list)->impl, ((void)((list)->base == *(datap)), (datap)),  \
+  gv_list_detach_(&(list)->impl, ((void)(&(list)->base == (datap)), (datap)),  \
                   (sizep), sizeof((list)->base[0]))
 
 #ifdef __cplusplus
