@@ -432,18 +432,8 @@ static bool write_subgs(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
 }
 
 static int agseqasc(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  Agedge_t **lhs = (Agedge_t **)x;
-  Agedge_t **rhs = (Agedge_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
+  Agedge_t *const *const lhs = x;
+  Agedge_t *const *const rhs = y;
     Agedge_t *e1 = *lhs;
     Agedge_t *e2 = *rhs;
 
@@ -468,7 +458,7 @@ static void write_edge(Agedge_t *e, GVJ_t *job, bool top, state_t *sp) {
 	gvprintf(job, "\"tail\": %d,\n", ND_gid(agtail(e)));
 	indent(job, sp->Level);
 	gvprintf(job, "\"head\": %d", ND_gid(aghead(e)));
-    	write_attrs((Agobj_t*)e, job, sp);
+    	write_attrs(&e->base, job, sp);
 	gvputs(job, "\n");
 	sp->Level--;
 	indent(job, sp->Level);
@@ -537,7 +527,7 @@ static void write_node(Agnode_t *n, GVJ_t *job, bool top, state_t *sp) {
 	indent(job, sp->Level);
 	gvputs(job, "\"name\": ");
 	stoj(agnameof(n), sp, job);
-    	write_attrs((Agobj_t*)n, job, sp);
+    	write_attrs(&n->base, job, sp);
 	gvputs(job, "\n");
 	sp->Level--;
 	indent(job, sp->Level);
@@ -684,7 +674,7 @@ static void write_graph(Agraph_t *g, GVJ_t *job, bool top, state_t *sp) {
     indent(job, sp->Level++);
     gvputs(job, "{\n");
     write_hdr(g, job, top, sp);
-    write_attrs((Agobj_t*)g, job, sp);
+    write_attrs(&g->base, job, sp);
     if (top) {
 	gvputs(job, ",\n");
 	indent(job, sp->Level);
