@@ -33,7 +33,7 @@ def run_raw(args: list[Union[Path, str]], **kwargs) -> Optional[Union[bytes, str
     """
 
     # dump the command being run for the user to observe if the test fails
-    print(f"+ {shlex.join(str(x) for x in args)}")
+    print(f"+ {shlex.join(str(x) for x in args)}", flush=True)
 
     is_stdout_decoded = kwargs.get("universal_newlines") or kwargs.get("text")
 
@@ -171,7 +171,7 @@ def compile_c(
     except subprocess.CalledProcessError:
         try:
             os.remove(dst)
-        except FileNotFoundError:
+        except (FileNotFoundError, PermissionError):
             pass
         raise
 
@@ -230,7 +230,7 @@ def dot(
         kwargs["input"] = source
 
     # dump the command being run for the user to observe if the test fails
-    print(f"+ {shlex.join(str(x) for x in args)}")
+    print(f"+ {shlex.join(str(x) for x in args)}", flush=True)
 
     proc = subprocess.run(args, stdout=subprocess.PIPE, check=True, **kwargs)
     return proc.stdout
@@ -318,6 +318,13 @@ def is_fedora() -> bool:
     return freedesktop_os_release().get("ID") == "fedora"
 
 
+def is_fedora_43() -> bool:
+    """is the current environment Fedora 43?"""
+    if not is_fedora():
+        return False
+    return freedesktop_os_release().get("VERSION_ID") == "43"
+
+
 def is_macos() -> bool:
     """is the current platform macOS?"""
     return platform.system() == "Darwin"
@@ -339,6 +346,18 @@ def is_static_build() -> bool:
     return False
 
 
+def is_ubuntu() -> bool:
+    """is the current environment Ubuntu?"""
+    return freedesktop_os_release().get("ID") == "ubuntu"
+
+
+def is_ubuntu_2510() -> bool:
+    """is the current environment Ubuntu 25.10?"""
+    if not is_ubuntu():
+        return False
+    return freedesktop_os_release().get("VERSION_ID") == "25.10"
+
+
 def remove_asan_summary(s: str) -> str:
     """
     Remove the “Suppressions used…” informational text Address Sanitizer prints.
@@ -382,6 +401,13 @@ def is_rocky_8() -> bool:
     return is_rocky("8")
 
 
+def is_rocky_10() -> bool:
+    """
+    is the current environment Rocky Linux 10?
+    """
+    return is_rocky("10")
+
+
 def has_sandbox() -> bool:
     """
     do we have an available sandbox mechanism?
@@ -396,6 +422,40 @@ def has_sandbox() -> bool:
     return False
 
 
+def plugin_version() -> tuple[int, int, int]:
+    """get the Graphviz plugin libtool revision, current, age"""
+
+    # treat configure.ac as the canonical source of this information
+    configure_ac = Path(__file__).resolve().parents[1] / "configure.ac"
+
+    # parse out the version components
+    current: Optional[int] = None
+    revision: Optional[int] = None
+    age: Optional[int] = None
+    with open(configure_ac, "rt", encoding="utf-8") as f:
+        for line in f:
+            if m := re.match(r"\s*GVPLUGIN_CURRENT\s*=\s*(?P<current>\d+)\s*$", line):
+                current = int(m.group("current"))
+                if revision is not None and age is not None:
+                    break
+                continue
+            if m := re.match(r"\s*GVPLUGIN_REVISION\s*=\s*(?P<revision>\d+)\s*$", line):
+                revision = int(m.group("revision"))
+                if current is not None and age is not None:
+                    break
+                continue
+            if m := re.match(r"\s*GVPLUGIN_AGE\s*=\s*(?P<age>\d+)\s*$", line):
+                age = int(m.group("age"))
+                if current is not None and revision is not None:
+                    break
+                continue
+    assert current is not None, "failed to parse plugin version"
+    assert revision is not None, "failed to parse plugin version"
+    assert age is not None, "failed to parse plugin version"
+
+    return current, revision, age
+
+
 def run_c(
     src: Path,
     args: list[Union[Path, str]] = None,
@@ -422,7 +482,7 @@ def run_c(
 
         # dump the command being run for the user to observe if the test fails
         argv = [exe] + args
-        print(f"+ {shlex.join(str(x) for x in argv)}")
+        print(f"+ {shlex.join(str(x) for x in argv)}", flush=True)
 
         input_bytes = None
         if input is not None:
